#pragma once
typedef float nvReal;


#include <vector>
#include <list>


class CShape;
class CPlaneShape;
class CJointsMngr;
struct VECTOR3;
class CVBody;
class CPhEngine;
class CPrimitive;
class CVirtualBodiesList;
class CPlaneShape;
class CJoint;
class CCylinderShape;
class CConvexShape;
class CBoxShape;
class CSphereShape;
class CTriMeshShape;
class CShapesMngr;

namespace nV
{

	class Shape;
	class PlaneShape; 
	class BoxShape;
	class SphereShape;
	class CylinderShape;
	class ConvexShape;
	class TriMeshShape;

	class ShapeMngr
	{
		friend class PhysicsWorld;

		::CShapesMngr* pShapeMngr;
		std::list<nV::Shape*> Shapes;
	protected:
		ShapeMngr(void);
		~ShapeMngr(void);

	public:

		// create different shapes
		PlaneShape* CreatePlane(nvReal nx,nvReal ny,nvReal nz,nvReal d);
		PlaneShape* CreatePlane(nvReal xPointOnPlane,nvReal yPointOnPlane,nvReal zPointOnPlane,nvReal xNormalOnPlane,nvReal yNormalOnPlane,nvReal zNormalOnPlane);
		BoxShape* CreateBox(nvReal xLength,nvReal yLength,nvReal zLength);
		SphereShape* CreateSphere(nvReal radius);
		CylinderShape* CreateCylinder(nvReal radius,nvReal height);
		ConvexShape* CreateConvex();
		TriMeshShape* CreateTriMesh();
	    
		// destroy a shape
		// rarely used
		void DestroyShape(Shape* pShape);

		// this destroy all the shapes, should be called at the destruction of the engine
		void DestroyAllShapes();
	};

	
	class Shape
	{
		friend class ShapeMngr;
		friend class Body;
		
	protected:

		CShape * pShape;

		Shape(CShape *);
		~Shape();

		
	public:

		enum eShapeType
		{
			PLANE=0,
			BOX=1,
			SPHERE=2,
			CYLINDER=3,
			CONVEX=4,
			TRIMESH=5
		};


	public:

		virtual nvReal GetBoundingRadius();
		virtual nvReal GetBoundingRadiusSqr();


		// get the type before u cast
		virtual Shape::eShapeType GetType();

	};

	
	class TriMeshShape : public Shape
	{
		friend class ShapeMngr;
		friend class Body;

		CTriMeshShape* pShape;

	protected:
		TriMeshShape(CTriMeshShape*);
		~TriMeshShape(void);
	public:

		virtual nvReal GetBoundingRadius();
		virtual nvReal GetBoundingRadiusSqr();


		// get the type before u cast
		virtual Shape::eShapeType GetType();


		// TriMeshShapeSpecific:
		void Begin();
		void End();
		void AddTriangle(nvReal* V1,nvReal* V2,nvReal* V3);

	};


	class SphereShape :
		public Shape
	{
		friend class ShapeMngr;
		friend class Body;

		CSphereShape* pShape;

	protected:

		SphereShape(CSphereShape*);
		~SphereShape(void);

	public:

		virtual nvReal GetBoundingRadius();
		virtual nvReal GetBoundingRadiusSqr();


		// get the type before u cast
		virtual Shape::eShapeType GetType();


		//////////////////////////////////////

		nvReal GetRadius();

	};


	class BoxShape :
		public Shape
	{
		friend class ShapeMngr;
		friend class Body;

		CBoxShape* pShape;
	protected:
		BoxShape(CBoxShape*);
		~BoxShape(void);
	public:

		virtual nvReal GetBoundingRadius();
		virtual nvReal GetBoundingRadiusSqr();


		// get the type before u cast
		virtual Shape::eShapeType GetType();

		void GetDimensions(nvReal &x,nvReal &y,nvReal &z);


	};


	class ConvexShape :
		public Shape
	{
		friend class ShapeMngr;
		friend class Body;

		CConvexShape* pShape;
	
	protected:
		ConvexShape(CConvexShape*);
		~ConvexShape();
	public:

		virtual nvReal GetBoundingRadius();
		virtual nvReal GetBoundingRadiusSqr();


		// get the type before u cast
		virtual Shape::eShapeType GetType();


		//////////////////////////////////////
		//ConvexShape Specifics:

		// a faster way to add vertices should be in this form
		// send a pointer to the first vertex
		// and the stride (in bytes) from one vertex to the next
		// data will be copied locally
		void AddVertices(void* pStartVertex,int stride,int vertexCount);

		// add a new edge, by putting 2 vertices, and one edge
		void AddEdge(nvReal* pVertexA,nvReal* pVertexB);

		// add a new face..
		// should create one face, and a number of edges= number of vertices, and add the vertices
		// we start with a triangle
		// the 3 vertices are in CCW order
		void AddTriangle(nvReal* pVertexA,nvReal* pVertexB,nvReal* pVertexC);

		// adds a list of triangles
		// using just 1 pointer
		// we take 3 x,y,z values by jumping by the stride vlues
		// each 3 vertices creates a new face
		void AddTriangles(void* pStartVertex,int stride,int numVerts);

		// adds a list of indexed triangles
		// using just 1 pointer to the vertices
		// and a pointer to the indices
		// we take 3 x,y,z values by jumping by the stride values
		// each 3 vertices creates a new face
		void AddIndexedTriangles(void* pStartVertex,int* pIndices,int stride,int indexCount);

		// add a triangles strip
		// same as the previous, but the vertices are added in a different way
		// each vertex takes the 2 previous ones, and inverts the order of them
		void AddTriangleStrip(void* pStartVertex,int stride,int vertexCount);

		// add an indexed triangles strip
		// same as the previous, but the vertices are added in a different way
		// each vertex takes the 2 previous ones, and inverts the order of them
		void AddIndexedTriangleStrip(void* pStartVertex,int* pIndices,int stride,int indexCount);

		// adds a triangle fan
		void AddTriangleFan(void* pStartVertex,int stride,int vertexCount);

		// adds an indexed triangle fan
		void AddIndexedTriangleFan(void* pStartVertex,int stride,int vertexCount);

		// these 2 function calls are essential 
		// vertices should only be added between a begin and an end
		void Begin();

		// call this when done with adding vertices
		void End();

	};


	class PlaneShape :
		public Shape
	{
		friend class ShapeMngr;
		friend class Body;

		CPlaneShape* pShape;
	protected:
		PlaneShape(CPlaneShape*);
		~PlaneShape(void);

	public:

		virtual nvReal GetBoundingRadius();
		virtual nvReal GetBoundingRadiusSqr();


		// get the type before u cast
		virtual Shape::eShapeType GetType();

	};


	class CylinderShape :
		public Shape
	{
		friend class ShapeMngr;
		friend class Body;

		CCylinderShape* pShape;

	protected:
		CylinderShape(CCylinderShape*);
		~CylinderShape(void);
	public:

		virtual nvReal GetBoundingRadius();
		virtual nvReal GetBoundingRadiusSqr();


		// get the type before u cast
		virtual Shape::eShapeType GetType();

		//////////////////////////////////////
		//CCylindeShape Specifics
		nvReal GetRadius();
		nvReal GetHeight();


	};

	class Joint
	{
		friend class JointMngr;

		CJoint * pJoint;
	protected:
		Joint(CJoint*);
		~Joint(void);
	};

	class Body;
	class Primitive;
	class Joint;

	class JointMngr
	{
		friend class PhysicsWorld;

		CJointsMngr * pJointsMngr;
		std::list<nV::Joint*> Joints;
	public:
		JointMngr(void);
		~JointMngr(void);

		nV::Joint* CreateBallJoint(nV::Body *pBdy1, nV::Body *pBdy2,
		nvReal xGlobalPoint,nvReal yGlobalPoint,nvReal zGlobalPoint);

		nV::Joint* CreateBallJoint(nV::Body *pBdy1,nvReal r1x,nvReal r1y,nvReal r1z,
						nV::Body *pBdy2,nvReal r2x,nvReal r2y,nvReal r2z);

		nV::Joint* CreateCarWheelJoint(nV::Body *pBdy1, nV::Body *pBdy2, 
		nvReal xGlobalPoint,nvReal yGlobalPoint,nvReal zGlobalPoint,
		nvReal xAxis,nvReal yAxis,nvReal zAxis,
		nvReal xSteeringAxis,nvReal ySteeringAxis,nvReal zSteeringAxis);

		nV::Joint* CreateContactJoint(nV::Primitive* pPrimitive1,nV::Primitive* pPrimitive2,VECTOR3 P,
		VECTOR3 B2toB1normal,nvReal overlappedDistance,bool isFirstPoint,bool IsOneFrameJoint);

		nV::Joint* CreateFixedJoint(Body* pBdy1,Body* pBdy2);

		nV::Joint* CreateFrictionJoint(const nV::Primitive* pPrim1,const nV::Primitive* pPrim2,const VECTOR3 &B2toB1normal,const VECTOR3 &NormalPerp1,const VECTOR3 &NormalPerp2,const VECTOR3 &P,const int NoOfFricitonPoints,const bool bIsFirst,const bool IsOneFrameJoint);

		nV::Joint* CreateHingeJoint(Body * pBdy1, Body *pBdy2, 
		nvReal xGlobalPoint,nvReal yGlobalPoint,nvReal zGlobalPoint,
		nvReal xAxis,nvReal yAxis,nvReal zAxis);

		nV::Joint* CreateStiffSpringJoint(Body* pBdy1,
				nvReal xR1,nvReal yR1,nvReal zR1,
				Body* pBdy2,
				nvReal xR2,nvReal yR2,nvReal zR2,
				nvReal L);

		void DestroyJoint(nV::Joint* pJ);

		void DeleteAllBodyJoints(nV::Body* pBody);
	};


	class Body;
	class Shape;

	class Primitive
	{
		friend class JointMngr;
		friend class Body;

		CPrimitive * pPrimitive;
		Body* pBody;
		Shape* pShape;

	protected:
		Primitive();
		Primitive(CPrimitive*, Body*, Shape*);
		~Primitive(void);
	public:

		void RotateAxisAngle(nvReal xAxis,nvReal yAxis,nvReal zAxis,nvReal angle);
	
		/* Setter & Getter */
		void SetElasticity(nvReal elasticity);
		nvReal GetElasticity();

		void SetFriction(nvReal friction);
		nvReal GetFriction();

		void SetData(void *pData);
		void* GetData();

		void SetLocalPosition(nvReal x,nvReal y,nvReal z);
		void GetLocalPosition(nvReal &x,nvReal &y,nvReal &z);
		void GetWorldPosition(nvReal &x,nvReal &y,nvReal &z);
		nvReal* GetWorldRotationMatrix();
		nvReal* GetLocalRotationMatrix();

		Body* GetBody();
		Shape*	GetShape();		//ezzay??
	};


	class Shape;
	class Primitive;
	class BodyMngr;

	class Body
	{
		friend class JointMngr;
		friend class BodyMngr;

		CVBody * pBody;
		BodyMngr* pManager;

	protected:
		Body(CVBody*, BodyMngr*);
		~Body(void);

	public:

		std::vector<nV::Primitive*> Primitives;

		Primitive* AddPlane(nvReal nx,nvReal ny,nvReal nz,nvReal d);
		Primitive* AddPlane(nvReal xPointOnPlane,nvReal yPointOnPlane,nvReal zPointOnPlane,nvReal xNormalOnPlane,nvReal yNormalOnPlane,nvReal zNormalOnPlane);
		Primitive* AddBox(nvReal mass,nvReal xLength,nvReal yLength,nvReal zLength);
		Primitive* AddSphere(nvReal mass,nvReal radius);
		Primitive* AddCylinder(nvReal mass,nvReal radius,nvReal height);
		Primitive* AddConvex(nvReal mass);
		Primitive* AddTriMesh(nvReal mass);

		Primitive* AddNewPrimitive(nV::Shape* pShap,nvReal mass,nvReal* pWorldPosition=0,nvReal* pWorldRotation=0);

		
		void RotateAxisAngle(	nvReal xAxis,nvReal yAxis,nvReal zAxis,nvReal angle);
		void Move( nvReal deltaX,nvReal deltaY,nvReal deltaZ);
		void AddPermanentForce(	nvReal xForceOnCenterOfMass,nvReal yForceOnCenterOfMass,nvReal zForceOnCenterOfMass);
		void AddPermanentForce(	nvReal xBodySpaceForce,nvReal yBodySpaceForce,nvReal zBodySpaceForce,
								nvReal xBodySpacePointOfAct,nvReal yBodySpacePointOfAct,nvReal zBodySpacePointOfAct);
		void AddOneFrameForce(	nvReal xForceOnCenterOfMass,nvReal yForceOnCenterOfMass,nvReal zForceOnCenterOfMass);
		void AddOneFrameForce(	nvReal xBodySpaceForce,nvReal yBodySpaceForce,nvReal zBodySpaceForce,
								nvReal xBodySpacePointOfAct,nvReal yBodySpacePointOfAct,nvReal zBodySpacePointOfAct);

		void AddOneFrameTorque( nvReal xTorque,nvReal yTorque,nvReal zTorque);
		void AddPermanentTorque( nvReal xTorque,nvReal yTorque,nvReal zTorque);

		/* Setter & Getter */
		nvReal* GetRotationMatrix();
		void GetPositon(nvReal &xWorldPosition,nvReal &yWorldPosition,nvReal &zWorldPosition);
		void SetPosition(nvReal xWorldPosition,nvReal yWorldPosition,nvReal zWorldPosition);
		void GetAngularVelocity(nvReal &xAngularVelocity,nvReal &yAngularVelocity,nvReal &zAngularVelocity);
		void SetAngularVelocity(nvReal xAngularVelocity,nvReal yAngularVelocity,nvReal zAngularVelocity);
		void GetLinearVelocity(nvReal &xLinearVelocity,nvReal &yLinearVelocity,nvReal &zLinearVelocity);
		void SetLinearVelocity(nvReal xLinearVelocity,nvReal yLinearVelocity,nvReal zLinearVelocity);
		void SetData(void* pUserData);
		void* GetData();
		nvReal GetMass();
		void GetCenterOfMass(nvReal &xWorldCenterOfMass,nvReal &yWorldCenterOfMass,nvReal &zWorldCenterOfMass);
		bool IsSleeping();
		bool IsStatic();

	};


	class Body;
	class PhysicsWorld;

	class BodyMngr
	{
		friend class PhysicsWorld;
		friend class Body;

		CVirtualBodiesList* pBodyList;
		std::list<Body*> Bodies;
		
		PhysicsWorld* pWorld;

	protected:
		BodyMngr(void);
		~BodyMngr(void);
	public:

		Body* Create();
		void Destroy(Body* pBody);
	};



	class Primitive;
	typedef bool (__cdecl * CollisionNotifyBackFnc)(void * pPrim1,void* pPrim2);

	class PhysicsWorld
	{
		CPhEngine * pEngine;
		//static CollisionNotifyBackFnc ColCallBack;
		//static bool CollideCallBack(::Primitive* p1,::Primitive* p2);
	public:

		BodyMngr BodyMngr;
		JointMngr JointMngr;
		ShapeMngr ShapeMngr;
		PhysicsWorld(void);
		~PhysicsWorld(void);

		void Reset();
		void RunStep(nvReal dtInSeconds);
		void RunStep(nvReal dtInSeconds,nvReal MaxDtInSeconds,int MaxNoOfTimeSegments);
		void RunUntil(unsigned long TimeInMilliseconds,nvReal MaxDtInSeconds = 0.01f,int MaxNoOfTimeSegments = 3);

		void SetCollisionNotify(CollisionNotifyBackFnc FnAddr);
		void SetGravity(nvReal xAcceleration,nvReal yAcceleration,nvReal zAcceleration);
		void SetNoOfSolvingIterations(int NoOfIterations);
		void SetObjectsAutoDeactivation(bool AutoDeactivateObjects);
	};
}