#ifndef _BSP_TREE_H_
#define _BSP_TREE_H_

/********************************************************
*	bsptree.h											*
*	declaration file to he BSP tree class				*
********************************************************/

#include "bsptypes.h"
#include "bitset.h"
#include "models/models.h"

#define BSP_EPSILON				0.03125f
#define BSP_COLLISION_RAY		0x00000000
#define BSP_COLLISION_SPHERE    0x00000001

// a simple structure will return the result of a ray trace collision detecions
struct sMoveData
{
    float     Fraction;              
    Vector3	EndPoint;              
    Vector3	CollisionNormal;        
    bool      StartOut;              
    bool      AllSolid;              
};


/********************************************************
*	class cBSPTree declaration							*
*	the BSP tree holds all the BSP search tree nodes and*
*	leafs and planes and the corresponding visibility	*
*	data, it used yo speed up the Rendering process		*
*	of a BSP file										*
********************************************************/
class cBSPTree	
{
public:

    //	this is used to find the visible leafs from a certain position
    cBitset			m_FacesDrawn;

    //	the nodes and leafs and all required bsp tree data
    tBSPNode		*	Nodes		;
    tBSPLeaf		*	Leafs		;
    tBSPPlane		*	Planes		;
    tBSPVisData			Visibility	;
    tBSPBrush		*	Brushes		;
    tBSPBrushSide	*	BrushSides	;
    tBSPShader		*	Shaders		;
    tBSPTexture		*	Textures	;

    int				*	LeafFaces	;
    int				*	LeafBrushes	;

    int			NumNodes		;
    int			NumLeafs		;
    int			NumPlanes		;
    int			NumLeafFaces	;
    int			NumBrushes		;
    int			NumLeafBrushes	;
    int			NumBrushSides	;
    int			NumShaders		;
    int			NumTextures		;

    //	data for collision detection and handling
    Vector3 m_CollisionStart;			//	where the tracing begins
    Vector3 m_CollisionEnd;				//	where the tarcing ends

    byte     m_MoveObjectType;			// This hold the Object Type (Ray / Sphere)
    float    m_MoveOffset;				// This hold the Object Offset (Ray = 0, Sphere = Radius)


    //	 the collision detection data structure, returned after each collision check
    sMoveData m_MoveData;

public:


    //	arrays for special faces (blended ..alpha..)and doors
    vector<sBSPFace*>		BlendedFaces;
    vector<sBSPFace*>		AlphaFaces;
    vector<sBSPFace*>		NormalFaces;
    vector<cMapDoor*>		Doors;

    //	constructor
    cBSPTree();

    //	deallocate the memory
    ~cBSPTree();

    //	get the index of the current leaf
    int		FindLeaf(const Vector3 &vPos);


    //	this will Render all polygon faces and setup the special ones
    void FilterFaces(const Vector3& Position,sBSPFace** m_ppFaces,vector<cMapDoor*>&	Doors);

    //	visibility checking function,inline for speed
    int		IsVisible(int current, int test)
    {
        if(!Visibility.m_pBitsets || current < 0) return 1;
        byte visSet = Visibility.m_pBitsets[(current*Visibility.bytesPerCluster) + (test >> 3)];
        return ( visSet & (1 << ((test) & 7)) );
    }

    // collision detection functions
    void CheckMoveByNode(int nodeIndex,float startFraction,float endFraction,Vector3& Start,Vector3& End);
    void CheckTouchBrush(tBSPBrush *brush);
    sMoveData& CheckSphereMove(Vector3& Start,Vector3& End,float Radius);
    sMoveData& CheckRayMove(Vector3& Start,Vector3& End);

    // the BSPFile class should be able to access everything here
    friend class cBSPFile;

    // this is used to construct the convex shape for a certain brush
    void CreateConvex(vector<Vector3>& verts,int brushIndex);

    // create all the convexes
    void CreateAllConvexes();
};
/*---------end of class cBSPTree declaration-----------*/
#endif // _BSP_TREE_H_