#ifndef _BSPTYPES_H_
#define _BSPTYPES_H_
/********************************************************
*	bsptypes.h											*
*	By SHady El Mously									*
*	this is a header file for all bsp lump types		*
********************************************************/
#include "../math/math.h"

// This is the number that is associated with a face that is of type "polygon"
#define FACE_POLYGON		1
#define FACE_PATCH			2
#define FACE_MESHVERTICES	3
#define FACE_BILLBOARD		4

#define MAX_TEXTURES 1000				// The maximum amount of m_Textures to load

typedef Plane tBSPPlane;
typedef int tBSPLeafFace;
typedef int tBSPLeafBrush;
typedef unsigned int tBSPMeshVertex;


/********************************************************
*	lump enumeration type								*
*	each type represent a lump inside the bsp file		*
********************************************************/
enum eLumps
{
    kEntities = 0,				// Stores player/object positions, etc...
    kTextures,					// Stores texture information
    kPlanes,				    // Stores the Splitting planes
    kNodes,						// Stores the BSP nodes
    kLeafs,						// Stores the leafs of the nodes
    kLeafFaces,					// Stores the leaf's indices into the faces
    kLeafBrushes,				// Stores the leaf's indices into the brushes
    kModels,					// Stores the info of world models
    kBrushes,					// Stores the brushes info (for collision)
    kBrushSides,				// Stores the brush surfaces info
    kVertices,					// Stores the level vertices
    kMeshVerts,					// Stores the model vertices offsets
    kShaders,					// Stores the shader files (blending, anims..)
    kFaces,						// Stores the faces for the level
    kLightmaps,					// Stores the lightmaps for the level
    kLightVolumes,				// Stores extra world lighting information
    kVisData,					// Stores PVS and cluster info (visibility)
    kMaxLumps					// A constant to store the number of lumps
};
/*----------------end of lumps enumeration-------------*/


/********************************************************
*	bsp header type delcaration							*
********************************************************/
struct tBSPHeader
{
    char strID[4];				// This should always be 'IBSP'
    int version;				// This should be 0x2e for Quake 3 files
}; 
/*--------------end of tBSPHeader declaration----------*/

/********************************************************
*	BSP lump struct										*
*	each lump starts in the file at offset				*
*	and its size in bytes is length						*
********************************************************/
struct tBSPLump
{
    int offset;					// The offset into the file for the start of this lump
    int length;					// The length in bytes for this lump
};
/*--------------end of tBSPLump declaration------------*/

/********************************************************
*	BSPVertex											*
*	each vertex for the bsp file is in the following	*
*	format												*
********************************************************/
struct tBSPVertex
{
    Vector3 vPosition;			// (x, y, z) position. 
    Vector2 vTextureCoord;		// (u, v) texture coordinate
    Vector2 vLightmapCoord;	// (u, v) lightmap coordinate
    Vector3 vNormal;			// (x, y, z) normal vector
    unsigned char color[4];				// RGBA color for the vertex 
};
/*------------end of tBSPVertex declaration------------*/

/********************************************************
*	BSP face											*
*	faces are renderable objects inside the map			*
*	which can be one of 4 types							*
*	and can have shaders								*
********************************************************/
struct tBSPFace
{
    int m_ID;				// The index into the texture array 
    int effect;					// The index for the effects (or -1 = n/a) 
    int type;					// 1=polygon, 2=patch, 3=mesh, 4=billboard 
    int startVertIndex;			// The starting index into this face's first vertex 
    int numOfVerts;				// The number of vertices for this face 
    int meshVertIndex;			// The index into the first meshvertex 
    int numMeshVerts;			// The number of mesh vertices 
    int lightmapID;				// The texture index for the lightmap 
    int lMapCorner[2];			// The face's lightmap corner in the iMage 
    int lMapSize[2];			// The size of the lightmap section 
    Vector3 lMapPos;			// The 3D origin of lightmap. 
    Vector3 lMapVecs[2];		// The 3D space for s and t unit vectors. 
    Vector3 vNormal;			// The face normal. 
    int size[2];				// The bezier patch dimensions. 
};
/*-------------end of tBSPFace declaration-------------*/

/********************************************************
*	tVector3i											*
*	a simple vector using integer values				*
********************************************************/
struct tVector3i
{
    int x, y, z;				// The x y and z position of our integer vector
};
/*-----------------end of tVector3i--------------------*/

/********************************************************
*	tBSPTexture											*
*	the bsp texture format struct						*
********************************************************/
struct tBSPTexture
{
    char strname[64];			// The name of the texture w/o the extension 
    int flags;					// The surface flags (unknown) 
    int contents;				// The content flags (unknown)
};
/*---------------end of tBSPTexture declaration--------*/

/********************************************************
*	tBSPLightMap										*
*	stores the lightmap color data						*
********************************************************/
struct tBSPLightmap
{
    unsigned char iMageBits[128][128][3];   // The RGB data in a 128x128 iMage
};
/*-------------end of tBSPLIghtMap declaration---------*/


/********************************************************
*	tBSPNode											*
*	a node in the bsp tree								*
********************************************************/
struct tBSPNode
{
    int plane;					// The index into the planes array 
    int front;					// The child index for the front and back nodes, resepctively
    int back;
    tVector3i min;				// The bounding box min position. 
    tVector3i max;				// The bounding box max position. 
};
/*---------------end of tBSPNode declaration-----------*/

/********************************************************
*	tBSPLeaf											*
*	a lead in the bsp tree								*
********************************************************/
struct tBSPLeaf
{
    int cluster;				// The visibility cluster 
    int area;					// The area portal 
    tVector3i min;				// The bounding box min position 
    tVector3i max;				// The bounding box max position 
    int leafface;				// The first index into the face array 
    int numOfLeafFaces;			// The number of faces for this leaf 
    int leafBrush;				// The first index for into the brushes 
    int numOfLeafBrushes;		// The number of brushes for this leaf 
};
/*---------------end of tBSPLeaf declaration-----------*/

/********************************************************
*	tBSPVisibility data									*
*	a large bit array for testing visibility			*
*	PVS													*
********************************************************/
struct tBSPVisData
{
    int numOfClusters;			// The number of clusters
    int bytesPerCluster;		// The amount of bytes (8 bits) in the cluster's bitset
    char *m_pBitsets;				// The array of bytes that holds the cluster bitsets				
};
/*----------end of tBSPVisData declaration-------------*/

/********************************************************
*	tBSPBrush											*
*	a brush is used to handle collision					*
*	each brush has a number of sides					*
********************************************************/
struct tBSPBrush 
{
    int brushSide;           // The starting brush side for the brush 
    int numOfBrushSides;     // Number of brush sides for the brush
    int textureID;           // The shader index 
}; 
/*----------end of tBSPBrush declaration---------------*/

/********************************************************
*	tBSPBrushSide declaration							*
*	used to resent a single side in a brush				*
********************************************************/
//	the brush side define a plane in the brush
struct tBSPBrushSide 
{
    int plane;              // The plane index
    int m_ID;				// The texture index
}; 
/*--------end of tBSPBrushFace declaration-------------*/

/********************************************************
*	struct tBSPModel									*
*	represents a movable object in the map				*
*	like a door											*
********************************************************/
struct tBSPModel 
{
    Vector3 min;           // The min position for the bounding box
    Vector3 max;           // The max position for the bounding box. 
    int faceIndex;          // The first face index in the model 
    int numOfFaces;         // The number of faces in the model 
    int brushIndex;         // The first brush index in the model 
    int numOfBrushes;       // The number brushes for the model
}; 
/*---------------end of tBSPModel declaration----------*/

/********************************************************
*	struct tBSPShader									*
*	used for redirecting to a shader file				*
********************************************************/
struct tBSPShader
{
    char strname[64];     // The name of the shader file 
    int brushIndex;       // The brush index for this shader 
    int unknown;          // This is 99% of the time 5
}; 
/*----------end of struct tBSPShader declaration-------*/

/********************************************************
*	tBSPLIght											*
*	carrying information about lights in the map		*
********************************************************/
struct tBSPLight
{
    byte ambient[3];     // This is the ambient color in RGB
    byte directional[3]; // This is the directional color in RGB
    byte direction[2];   // The direction of the light: [phi,theta] 
}; 
/*-------------end of tBSPLight declaration------------*/

/********************************************************
*	struct sLevelVertex									*
*	vertex format used for Rendering the bsp level		*
********************************************************/
struct	sLevelVertex
{
    Vector3	Position;
    Vector2	TexCoord;
    Vector2 LightMapCoord;
};
/*-----------end of sLevelVertex declaration-----------*/

#endif // _BSPTYPES_H_