#pragma once

/*
Stream output for D3D9

By: Eyal Teler

Note: This is an initial implementation, which lacks any error checking and other goodies.

License: This code may be used freely. If you mention my name in your code, that'd be nice.

Usage:

Shader code: See StreamOut.fx for an example

C++ code:

Create a CStreamOutput object with the maximum number of vertices you wish to output.

Use CreateOutputPositionsVB to create a vertex buffer
	(it contains the output locations that match each vertex)

Create a vertex declaration which looks something like this:
	D3DVERTEXELEMENT9 decl[] = 
	{
		{ 0, 0, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
		{ 1, 0, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },
		D3DDECL_END( )
	};
The first stream matches your standard vertex buffer, and references the data that's
going to be used.
The second stream will have the output positions VB set to it.

Set the streams and set the "stream output" shaders (like those in StreamOut.fx).
Render using DrawPoints. This will run the vertex shader on each vertex in turn,
and output them.

Lock to access the results.


Note that the output buffer is suitable for further processing using R2VB on ATI cards.
All that's needed is to add	D3DUSAGE_DMAP when creating the render target. (And, of
course, implementing the rest of the R2VB code.)
*/





#include <d3d9.h>

struct C2DPos
{
	float x,y;
};


// CStreamOutput implements stream output for D3D9 in the form of a floating point texture
// that is written to vertex by vertex, each pixel corresponding to output of one vertex.
// The class manages the texture and provides both a "vertex output position" vertex buffer
// that enables this pixel by pixel writing, and a conversion of indices to a similar vertex
// buffer of positions, in order to allow reading the data in the output texture by a pixel
// shader acting as a geometry shader.
class CStreamOutput
{
public:
	CStreamOutput(IDirect3DDevice9 * inDevice, UINT inMaxVertices, D3DFORMAT inFormat = D3DFMT_A32B32G32R32F);
	~CStreamOutput();

	void OnDeviceLost();
	void OnDeviceReset();

	void DrawPoints(UINT inStartVertex, UINT inVertexCount);

	void Lock(void ** outData);
	void Unlock();

	void CreateOutputPositionsVB(IDirect3DVertexBuffer9 ** outVB);

private:
	IDirect3DDevice9 * mDevice;
	D3DFORMAT mFormat;
	UINT mMaxVertices;
	UINT mTextureWidth;
	UINT mTextureHeight;

	IDirect3DTexture9 * mOutputTexture;
	IDirect3DSurface9 * mOutputSurface; // Surface of the output texture, for ease of SetRenderTarget
	IDirect3DSurface9 * mMemCopy; // For locking
};
