/********************************************************
*	graphics subsystem.h								*
*	By Shady El Mously									*
*	Part of the new Twister3d game engine				*
*	this file implementa the graphics subsystem			*
*	using OpenGL										*
********************************************************/
#ifndef _GRAPHICS_H_
#define _GRAPHICS_H_

#include "../../system/engine.h"
#include "../../system/window.h"
#include "../text/texturefont.h"
#include "frustum.h"


#define WM_CHANGESETTINGS (WM_USER+1)		//	define for a special windows message

// the matrix modes
#define TWISTER_MATRIXMODE_WORLD        0x00000000
#define TWISTER_MATRIXMODE_VIEW         0x00000001
#define TWISTER_MATRIXMODE_PROJECTION   0x00000002
#define TWISTER_MATRIXMODE_TEXTURE      0x00000003
#define TWISTER_MATRIXMODE_TEXTURE0     0x00000003
#define TWISTER_MATRIXMODE_TEXTURE1     0x00000004

class cTextureFont;
struct sMesh;

/********************************************************
*	class cGraphicsSubSystem							*
* represent the graphics layer, hiding directx8,		*
*	or opengl											*
********************************************************/
class cGraphicsSubSystem : public singleton<cGraphicsSubSystem>
{
protected:

    HDC			m_HDC;	//	main window device context

    // the texture font used 
    cTextureFont*       m_pTextureFont;


    // a function to compute the frustum using the view and projection matrices
    void ComputeFrustum(float* proj,float* modl);

    // the matrix mode currently used
    unsigned int m_MatrixMode;

    // the immedate mode mesh and rendering mode
    unsigned int    m_IMVertexFormat;
    sMesh*          m_pIMMesh;


public:

    cWindow*	m_pWindow;	//	pointer to main graphics window

    bool	m_bCanMultiTexture		;	//	can do multim_Textures?


    int		m_StartX				;	// the x start location of the window (0 when full screen)
    int		m_StartY				;	// the y start location of the window (0 when full screen)
    int		m_Width					;	// the width  of the main graphics window
    int		m_Height				;	// the height of the main graphics window
    float   m_FOVAngle              ;
    float   m_NearClip              ;
    float   m_FarClip               ;

    // the furstum..
    sFrustumData m_Frustum;


    //	default constructor
    cGraphicsSubSystem();

    //	destructor
    virtual ~cGraphicsSubSystem()
    {		DestroyWindow();	}



    //	Create the window
    void MakeWindow(sScreenMode& Mode);

    //	Destroy the window
    void DestroyWindow();

    //	this will Initialize openGL  and its window
    void Initialize(sScreenMode& Mode);

    // internal initialization function
    virtual void InitGraphics(sScreenMode& Mode)=0;

    //	shuts down OpenGL
    virtual void ShutDownGraphics()=0;

    //	this will resize the view port
    virtual void Resize(int Width,int Height)=0;

    // swap the front and back buffers
    virtual void SwapBuffers()=0;

    //get the device...
    virtual void * GetDevice(){return 0;}

    //	change the display settings
    void ChangeSettings(unsigned int Width,unsigned int Height,unsigned int depth,bool Windowed);

    //	get the main window handle
    HWND	GetMainHWND()
    {
        return m_pWindow->GetHwnd();
    }

    // use this to set another DC where we need openGL, used mostly in MFC applications
    void SetDC(HDC dc)
    {	m_HDC=dc;	}


    // clear the scene
    virtual void Clear()=0;

    // begin rendering here..
    virtual void BeginScene()=0;

    // call this when rendering is done
    virtual void EndScene()=0;

    // set the view matrix
    virtual void SetViewMatrix(float* pMatrix)=0;

    // set the world matrix
    virtual void SetWorldMatrix(float* pMatrix)=0;

    // set the projection matrix
    virtual void SetProjectionMatrix(float* pMatrix)=0;

    // set the texture matrix
    virtual void SetTextureMatrix(float* pMatrix)=0;

    // calculate the planes of the frustum
    virtual void CalculateFrustum()=0;

    // get the world matrix
    virtual void GetWorldMatrix(float* pMatrix)=0;

    // get the View matrix
    virtual void GetViewMatrix(float* pMatrix)=0;

    // get the projection matrix
    virtual void GetProjectionMatrix(float* pMatrix)=0;

    // get the texture matrix
    virtual void GetTextureMatrix(float* pMatrix)=0;

    // get the texture matrix of a certain stage
    virtual void GetTextureMatrix(float* pMatrix, int stage)=0;


    // check visibility of a cube
    bool CanViewPoint( float x, float y, float z);

    // check visibility of a cube
    bool CanViewCube( float x, float y, float z, float size);

    // check visibility of a box
    bool CanViewBox( float x, float y, float z, float x2, float y2, float z2);

    // check visibility of a sphere
    bool CanViewSphere( float x, float y, float z, float radius);

    // render the 3 main axis
    virtual void RenderAxis(float length);

    // matrix stack pushing (only working on the world matrix now)
    virtual void PushMatrix()=0;

    // matrix stack pushing (only working on the world matrix now)
    virtual void PopMatrix()=0;

    // translation, only world matrix is changed
    virtual void Translate(float x,float y,float z)=0;

    // rotation, only world matrix is changed
    virtual void Rotate(float angle,float x,float y,float z)=0;

    // scale, only world matrix is changed
    virtual void Scale(float sX,float sY,float sZ)=0;

    // matrix multiplication (only for world 
    virtual void MultMatrix(float* pMatrix)=0;

    // matrix loading
    virtual void LoadMatrix(float* pMatrix)=0;

    // matrix identity loading
    virtual void LoadIdentity()=0;

    // render a box
    virtual void RenderBox(float sizeX,float sizeY,float sizeZ)=0;

    // render a sphere
    virtual void RenderSphere(float radius)=0;

    // render a teapot?/
    virtual void RenderTeaPot()=0;

    // camera look at function
    virtual void LookAt(float* pLocation,float* pDirection,float* pUpVector)=0;

    // enable lighting
    virtual void EnableLighting()=0;

    // disable lighting
    virtual void DisableLighting()=0;

    // enable alpha
    virtual void EnableAlphaTest()=0;

    // disable alpha
    virtual void DisableAlphaTest()=0;

    // enable depth test
    virtual void EnableDepthTest()=0;

    // disable depth test
    virtual void DisableDepthTest()=0;

    // enable blending
    virtual void EnableBlending()=0;

    // disable blending
    virtual void DisableBlending()=0;

    // set the matrix mode using this
    virtual void SetMatrixMode(unsigned int mode)=0;

    // set the matrix mode using this
    unsigned int GetMatrixMode()
    {    return m_MatrixMode;  }




    // writing some text
    void WriteText(float x,float y,char* pText)
    {
        m_pTextureFont->Print(x,y,pText);
    }



    // entering 2d mode
    virtual void Begin2D()=0;

    //exiting 2d
    virtual void End2D()=0;

    // enable Enable sphere map automatic generation of texture coordinates on a certain stage
    virtual void EnableSphereMap()=0;

    // disable the sphere map
    virtual void DisableSphereMap()=0;

    // enable Enable sphere map automatic generation of texture coordinates on a certain stage
    virtual void EnableSphereMap(int stage)=0;

    // disable the sphere map
    virtual void DisableSphereMap(int stage)=0;

    // enable texturing on first stage
    virtual void EnableTexturing()=0;

    // Disable texturing on first stage
    virtual void DisableTexturing()=0;

    // enable rendering in wireframe
    virtual void EnableWireFrame()=0;

    // Disable rendering in wireframe
    virtual void DisableWireFrame()=0;

    // set perspective projection using this
    virtual void SetPerspectiveProjection(float fov,float ratio,float minZ,float maxZ)=0;

    // set orthogonal projection using this
    virtual void SetOrthogonalProjection(float startX,float endX,float startY,float endY,float nearZ,float farZ)=0;

    // compute a ray in the world using the screen coordinates
    virtual void ComputeMouseRay(int x,int y,float* pRaySource,float* pRayDirection)=0;

    // setiing line width
    virtual void SetLineWidth(float width)=0;

    // changing the clear color
    virtual void SetClearColor(float red,float green,float blue,float alpha)=0;

    //	main engine is friend into here
    friend class cTwisterEngine;



    // begin a rendering session
    void Begin(int renderMode);

    // immediate mode rendering functions
    void Vertex3f(float x,float y,float z);
    void Vertex3fv(float* pData);
    void Normal3f(float x,float y,float z);
    void Normal3fv(float* pData);
    void TexCoord2f(float s,float t);
    void TexCoord2fv(float *pData);
    void Color3f(float r,float g,float b);
    void Color3fv(float* pData);
    
    // end a rendering session
    void End();
};
//	the Graphics Subsystem object is accessed through this global function
inline cGraphicsSubSystem	*&	Graphics		(void){return cGraphicsSubSystem::GetSingletonPtr();}

cGraphicsSubSystem	* NewGraphicsGL();
cGraphicsSubSystem	* NewGraphicsDX();


/*-------------end of class cGraphicsSubSystem---------*/
#endif //_GRAPHICS_H_