#ifndef _TWISTER_MESHCOMPILER_H_
#define _TWISTER_MESHCOMPILER_H_

#include "../../util/util.h"        // for the singleton 

// vertex format defines
#define MESH_VERTEXFORMAT_POSITION      0x00000001
#define MESH_VERTEXFORMAT_POSITION3D    0x00000001
#define MESH_VERTEXFORMAT_POSITION2D    0x00000002
#define MESH_VERTEXFORMAT_POSITION4D    0x00000004
#define MESH_VERTEXFORMAT_TEXCOORD      0x00000010
#define MESH_VERTEXFORMAT_TEXCOORD0     0x00000010
#define MESH_VERTEXFORMAT_TEXCOORD1     0x00000020
#define MESH_VERTEXFORMAT_TEXCOORD2     0x00000040
#define MESH_VERTEXFORMAT_TEXCOORD3     0x00000080
#define MESH_VERTEXFORMAT_TEXCOORD4     0x00000100
#define MESH_VERTEXFORMAT_TEXCOORD5     0x00000200
#define MESH_VERTEXFORMAT_TEXCOORD6     0x00000400
#define MESH_VERTEXFORMAT_TEXCOORD7     0x00000800
#define MESH_VERTEXFORMAT_NORMAL        0x00001000
#define MESH_VERTEXFORMAT_COLOR         0x00002000

// render modes
#define MESH_RENDERMODE_TRIANGLES       0x00000001
#define MESH_RENDERMODE_TRIANGLEFAN     0x00000002
#define MESH_RENDERMODE_TRIANGLESTRIP   0x00000003
#define MESH_RENDERMODE_LINESTRIP       0x00000004
#define MESH_RENDERMODE_LINELIST        0x00000005
#define MESH_RENDERMODE_POINTS          0x00000006



// given a certain given format, this one will count the number of components
// (number of floating point variables)
int CountVertexComponents(unsigned int vertexFormat);


// an interface to index buffers
struct sIndexBuffer
{
    sIndexBuffer() {}
    virtual ~sIndexBuffer() {}
};

// an interface to vertex buffer
struct sVertexBuffer
{
    virtual void Render()=0;
    virtual void Render(sIndexBuffer*)=0;
    virtual void Render(sIndexBuffer*,int startIndex,int numVerts)=0;
    virtual void Render(sIndexBuffer*,int startVertex,int startIndex,int numVerts)=0;
    virtual void RenderFan(int startVertex,int numVerts)=0;
    virtual void RenderTriangles(int startVertex,int numVerts)=0;
    virtual void RenderStrip(int startVertex,int numVerts)=0;
    virtual void Enable()=0;
    virtual void Disable()=0;

    virtual void Lock()=0;
    virtual void Unlock()=0;
    virtual float* GetVertex(unsigned int vertexIndex)=0;

    sVertexBuffer() {}
    virtual ~sVertexBuffer() {}
};

// inteface for a material, all colors should be 4 floats
struct sInternalMaterial
{
    sInternalMaterial() {}
    virtual~ sInternalMaterial() {}
    virtual void    SetDiffuse(float* pColor)=0;
    virtual float*  GetDiffuse()=0;
    virtual void    SetAmbient(float* pColor)=0;
    virtual float*  GetAmbient()=0;
    virtual void    SetSpecular(float* pColor)=0;
    virtual float*  GetSpecular()=0;
    virtual void    SetEmissive(float* pColor)=0;
    virtual float*  GetEmissive()=0;
    virtual void    SetPower(float power)=0;
    virtual float   GetPower()=0;
    virtual void    Activate()=0;
    virtual void    Deactivate()=0;
};


// interface for a light source, all colors are 4 floats
struct sInternalLight
{
    static int		m_NumLights;					//	base starts from zero
    int             m_ID;

    sInternalLight() {m_ID=m_NumLights;m_NumLights++;}
    virtual~ sInternalLight() {m_NumLights--;}
    virtual void    SetDiffuse(float* pColor)=0;
    virtual float*  GetDiffuse()=0;
    virtual void SetAmbient(float* pColor)=0;
    virtual float* GetAmbient()=0;
    virtual void SetSpecular(float* pColor)=0;
    virtual float* GetSpecular()=0;

    virtual void Activate()=0;
    virtual void Deactivate()=0;
};


// interface for a directional light source
struct sInternalLightDirectional : public sInternalLight
{
    virtual void SetDirection(float* pDirection)=0;
    virtual float* GetDirection()=0;
};



// interface for a texture
struct sInternalTexture
{
    sInternalTexture(){}
    virtual ~sInternalTexture(){}

    // bind it here
    virtual void Bind()=0;

    // unbind it using this
    virtual void UnBind()=0;

    // bind it here on a certain texture stage
    virtual void Bind(int stage)=0;

    // unbind it using this on a certain stage
    virtual void UnBind(int stage)=0;
};


struct sShaderConstant
{
    // setting a 4x4 matrix 
    virtual void SetMatrix(const char* constantName,const float* pData);
    // setting a triplet vector
    virtual void SetVector(const char* constantName,const float* pData);
    // setting an integer
    virtual void SetInteger(const char* constantName,int value);
};

struct sShaderInterface
{
    sShaderInterface(){}
    virtual ~sShaderInterface(){}

    virtual void  Activate()=0;
    virtual void  DeActivate()=0;
    virtual bool  Compile(char* pFileName)=0;
    virtual char* GetError()=0;

    // setting a 4x4 matrix 
    virtual void SetMatrix(const char* constantName,const float* pData)=0;
    // setting a quad vector
    virtual void SetVector(const char* constantName,const float* pData)=0;
    // setting an integer
    virtual void SetInteger(const char* constantName,int value)=0;
    // setting a float
    virtual void SetFloat(const char* constantName,float value)=0;
    // get the texture stage for a certain sampler
    virtual int GetSamplerStage(char* pSamplerName)=0;

    // create an accesible constant
//    virtual sShaderConstant* GetConstant()=0;
};



class cMeshCompiler: public singleton<cMeshCompiler>
{
public:

    // take vertex data
    // and returns what will be needed later to render them all
    // or a range of them
    virtual sVertexBuffer* CreateVertexBuffer(float* pVertexData,unsigned int vertexFormat,unsigned int vertexCount,unsigned int renderMode)=0;

    // create an index buffer
    virtual sIndexBuffer* CreateIndexBuffer(unsigned int * pIndices,int indexCount,unsigned int renderMode,unsigned int vertexCount)=0;

    // create an internal texture from a file
    virtual sInternalTexture* CreateInternaltexture(const char* pFileName)=0;

    // create an internal texture from a raw data
    virtual sInternalTexture* CreateInternaltexture(const char* pData,short sizeX,short sizeY)=0;

    // create a material 
    virtual sInternalMaterial* CreateInternalMaterial()=0;

    // create a directional light
    virtual sInternalLightDirectional* CreateDirectionalLight()=0;


    // create a vertex shader interface
    virtual sShaderInterface* CreateVertexShader(char* pFileName)=0;

    // create a pixel shader interface
    virtual sShaderInterface* CreatePixelShader(char* pFileName)=0;
};

//	public access to the subsystem is using the function Simulation()
inline cMeshCompiler	*&	MeshCompiler(void)	{return cMeshCompiler::GetSingletonPtr();}


#endif //_TWISTER_MESHCOMPILER_H_