/********************************************************
*	matrix4.h											*
*	By Shady El Mously									*
*	main header for the 3d matrix						*
********************************************************/
#ifndef _MATRIX4_H_
#define _MATRIX4_H_

#include "vector3.h"
#include "vector4.h"
#include "segment.h"
#include "matrix3.h"



/********************************************************
*	struct matrix4 delcaration							*
*	represnts a 3d transformation matrix				*
*	which is the basis of all 3d operations				*
********************************************************/
struct Matrix4  
{
	//	3 different repreasentations for the same area in memory
    union 
	{
        struct 
		{
            float	_11, _12, _13, _14;
            float	_21, _22, _23, _24;
            float	_31, _32, _33, _34;
            float	_41, _42, _43, _44;
        };
        float	m[4][4];		//	can be accesses as a 3d array 4*4				(familar with direct3d)
		float   d[16];			//	can be accessed as a single 16 floats arrays	(OpenGL familiar)
    };

	//	default constructor ( no default values )
	Matrix4()
	{
		// Do nothing.  
	}


	//	ugly constructor, may be used for initializing static matrices
	Matrix4(
		float in11, float in12, float in13, float in14,
        float in21, float in22, float in23, float in24,
        float in31, float in32, float in33, float in34,
        float in41, float in42, float in43, float in44 )
	{
        _11 = in11; _12 = in12; _13 = in13; _14 = in14;
        _21 = in21; _22 = in22; _23 = in23; _24 = in24;
        _31 = in31; _32 = in32; _33 = in33; _34 = in34;
        _41 = in41; _42 = in42; _43 = in43; _44 = in44;
	}

	//	make it an identity matrix
	void MakeIdent();

	//	some static matrices (zero, identity)
	static const Matrix4 Identity;
	static const Matrix4 Zero;
	static const Matrix4 CubicBezier;		//	the basis matrix for cubic bezier curves
	static const Matrix4 QuadricBezier;		//	the basis matrix for quadric bezier curves

	//	multiplyting two matrices together

	//	first case.. multiplyign teo matrice and string the result..
	void Multiply(const Matrix4& A,const Matrix4& B);

	// rotation about any axe in space.. very useful
	void IsAxisRotation( const Vector3& inAxis, float angle );
	static Matrix4 AxisRotation( const Vector3& inAxis, float angle );

	// rotate the matrix around the 3 axis alltogether
	void Rotate( float yaw, float pitch, float roll );

	// static function to generate an inverse of a transformation matrix
	void IsInverse( const Matrix4& in );
	static Matrix4 Inverse( const Matrix4& in );


	// get the location
	inline Vector3&	GetLoc()
	{	return *(Vector3*)&_41;		}
	// get local X axis
	inline Vector3&	GetLocalX()
	{	return *(Vector3*)&_11;		}
	// get local Y axis
	inline Vector3&	GetLocalY()
	{	return *(Vector3*)&_21;		}
	// get local Z axis
	inline Vector3&	GetLocalZ()
	{	return *(Vector3*)&_31;		}

    // scale it by a certain factor
    void Scale(float sx,float sy,float sz);


    inline void SetRotationFromMatrix3(const Matrix3& source)
    {
        _11=source._11;_21=source._21;_31=source._31;
        _12=source._12;_22=source._22;_32=source._32;
        _13=source._13;_23=source._23;_33=source._33;
    }



};
/*---------------end of matrix4 declaration------------*/

/**
 * Specialization for multiplication by a point3 structure.
 * in a mathematically perfect world, we would construct
 * a 1x4 matrix for the point, and then multiply the 1x4
 * and the 4x4 matrices toegether.  However, this performs
 * much better.
 */
inline const Vector3 operator*( const Matrix4 &a, const Vector3 &b)
{
	return Vector3(
		b.x*a._11 + b.y*a._21 + b.z*a._31 + a._41,
		b.x*a._12 + b.y*a._22 + b.z*a._32 + a._42,
		b.x*a._13 + b.y*a._23 + b.z*a._33 + a._43
	);
};

inline const Vector3 operator*(const Vector3 &a, const Matrix4 &b)
{
	return b*a;
};


/**
 * Specialization for multiplication by a point4 structure.
 */
inline const Vector4 operator*(const Matrix4 &a, const Vector4 &b)
{
	return Vector4(
		b.x*a._11 + b.y*a._21 + b.z*a._31 + b.w*a._41,
		b.x*a._12 + b.y*a._22 + b.z*a._32 + b.w*a._42,
		b.x*a._13 + b.y*a._23 + b.z*a._33 + b.w*a._43,
		b.x*a._14 + b.y*a._24 + b.z*a._34 + b.w*a._44
	);
};

inline const Vector4 operator*(const Vector4 &a, const Matrix4 &b)
{
	return b*a;
};


//returns matrix4*lineSeg3
inline Segment operator*(Segment const &b,Matrix4 const &a)
{
	return Segment( b.v0 * a, b.v1 * a );
};
//returns matrix4*lineSeg3
inline Segment operator*(Matrix4 const &a,Segment const &b)
{
	return Segment( b.v0 * a, b.v1 * a );
};




#endif // _MATRIX4_H_
