#ifndef _PLANE3D_H_
#define _PLANE3D_H_


/********************************************************
*	plane.h												*
*	By Shady El Mously									*
*	header file for the plane data type					*
********************************************************/

#include "vector3.h"
#include "polygon.h"
#include "sphere.h"
#include "segment.h"
#include "matrix4.h"
#include <vector>
using std::vector;

// Defines the three p/* ossible locations of a point in 
// relation to a plane
enum ePointLoc
{
	ptFront,
	ptBack,
	ptCoplanar
};

// Defines the four possible locations of a point list in 
// relation to a plane.  A point list is a more general
// example of a polygon.
enum ePListLoc
{
	plistFront,
	plistBack,
	plistSplit,
	plistCoplanar
};

// same idea but for bounsinf spheres
enum eSphereLoc
{
	sphereFront, // Sphere was entirely in front
	sphereBack, // Sphere was entirely in back
	sphereCoplanar // Sphere was partially on either side
};


/********************************************************
*	Plane data type declaration							*
*	represents a plane in 3d							*
* and has various different functions					*
********************************************************/
struct Plane
{
	Vector3 n; // Normal of the plane
	float d; // Distance along the normal to the origin 

	Plane( float nX, float nY, float nZ, float D) : n( nX, nY, nZ ), d( D )
	{
		// All done.
	}
	Plane( const Vector3& N, float D) : n( N ), d( D )
	{
		// All done.
	}

	// Construct a plane from three 3-D points
	Plane( const Vector3& a, const Vector3& b, const Vector3& c)
	{
		n = (b-a)^(c-a);
		n.Normalize();
		d = -(n*a);
	}


	// Construct a plane from a normal direction and a point on the plane
	Plane( const Vector3& norm, const Vector3& loc):    n( norm ), d( -(norm*loc) )
	{
		// All done
	}

	
	// Construct a plane from a polygon
	Plane( const sPolygon<Vector3>& poly )
	{
		Vector3 a = poly.plist[0];
		Vector3 b = poly.plist[1];
		Vector3 c = poly.plist[2];

		n = (b-a)^(c-a);
		n.Normalize();
		d = -(n*a);
	}

	Plane()
	{
		// Do nothing
	}

	// all constructors defined..


	// now for some functions which relates planes to other things
	
	// tests a point against this plane
	ePointLoc TestPoint( const Vector3& point ) const
	{
		float dp = (point * n) + d;

		if(dp > EPSILON)
		{
			return ptFront;
		}
		if(dp < -EPSILON )
		{
			return ptBack;
		}
		return ptCoplanar; // it was between EP and -EP
	}

	// can test lines, polys, and bboxes.
	ePListLoc TestPolygon( const sPolygon<Vector3> &poly ) const
	{
		return TestPoints( poly.plist, poly.numelem );
	}

	// can test lines, polys, and bboxes.
	ePListLoc TestPoints(Vector3 *list, int num) const;

	// now for some function to cut things.. between sides of a plane..
	// first one is to find the intersection point if we link 2 points on both sides of a plane
	const Vector3 Split(const Vector3 &a, const Vector3 &b) const
	{
		float aDot = (a * n);
		float bDot = (b * n);
		float scale = ( -d - aDot ) / ( bDot - aDot );
		return a + (scale * (b - a));
	}

	// this will bring the point to the plane..
	const Vector3 Project(const Vector3& pt )
	{
		float distToPlane = (pt * n);
		return Vector3( pt - (d + distToPlane)*n );
	}


	
	// this will clip a polygon to another one.. in front of the plane
	bool Clip( const sPolygon<Vector3> &in,sPolygon<Vector3> *out) const;

	// this will clip a polygon to another one.. in front of the plane
	// the last parameter will be zero if the result is the same as the input
	bool Clip(vector<Vector3>& polyIn,vector<Vector3>& polyOut);


	// this one will split a polygon into 2 polygons.. one in front and one in back..
	bool Split(sPolygon<Vector3> const &in,sPolygon<Vector3> *pFront,sPolygon<Vector3> *pBack ) const;


	// this will flip the orientation of the plane
	void Flip()
	{
		n = -n;
		d = -d;
	}
	
	// this will split a segment into 2 lines
	// back on front.. return false on failure..
	bool Split( const Segment &sin,Segment *front,Segment *back ) const;


	// testing a sphere with the plane..
	// front.. back.. or partially here and there
	eSphereLoc TestSphere( const Sphere& sphere ) const;

	//	this will will get the reflected vector on the plane
	//	from andre lamothe
	Vector3 Reflect(Vector3& I);

	//	normalize the plane
	void Normalize();

	// intersect a segment with a plane... getting a point
	Vector3 Intersect(Segment& line);

	// intersect a plane with a nother plane, getting a segment
	Segment Intersect(Plane& other);

	// transform using a transformation matrix
	Plane Transform(Matrix4& matrix);
};
// this small overload to test equality of planes
inline bool operator==(Plane const &a,Plane const &b)
{
	if( fabs(a.d-b.d) < EPSILON )
		if( a.n == b.n )
			return true;
	return false;
};
/*----------------end of Plane declaration-------------*/
#endif // _PLANE3D_H_