/********************************************************
*	vector3.h											*
*	By Shady El Mously									*
*	part of the twister game engine						*
*	this is the declaration file fo the vector3 struct	*
*	the main 3d vector object used in the engine		*
********************************************************/

#ifndef _VECTOR3_H_
#define _VECTOR3_H_

#include <math.h>	//	for sqrtf
#include <assert.h>
#include "globals.h"

struct Vector4;

/********************************************************
*	struct Vector3 declaration							*
*	the main 3d vector									*
********************************************************/
struct Vector3
{
	union
	{
		float v[3];
		struct
		{
			float x,y,z;
		};
	};

	//	default constructor (empty)
	Vector3()		{}

	//	construct with 3 floats
	Vector3(float a,float b,float c):x(a),y(b),z(c){}

	//	construct with a vector4
	Vector3(Vector4& v);

	//	assigning values
	inline void Assign(float X,float Y,float Z)
	{
		x=X;
		y=Y;
		z=Z;
	}

	//	an add function which is better than the + operator (no constructor called)
	inline void Add(const Vector3& p1,const Vector3& p2)
	{
		x=p1.x+p2.x;
		y=p1.y+p2.y;
		z=p1.z+p2.z;
	}
	
	//	a subtract function which is better than the + operator (no constructor called)
	inline void Subtract(const Vector3& p1,const Vector3& p2)
	{
		x=p1.x-p2.x;
		y=p1.y-p2.y;
		z=p1.z-p2.z;
	}

	//	get the magnitude squared of the vector
	inline float MagSquared()
	{
		return x*x+y*y+z*z;
	}

	//	magnitude of the vector
	inline float Mag()
	{	return sqrtf(x*x+y*y+z*z);	}

	//	making the vector a normal vector
	inline void Normalize()
	{
		float M=1.0f/sqrtf(x*x+y*y+z*z);
		assert(M && "vector magnitude is zero, cannot normalize!");
		x=x*M;
		y=y*M;
		z=z*M;
	}


	//	get a normalized vector from this object
	const Vector3 Normalized() const 
	{
		Vector3 Ret=*this;
		Ret.Normalize();
		return Ret;
	}

	//	dot product..return a scalar quantity
	inline float Dot(Vector3& a)
	{	return x*a.x+y*a.y+z*a.z;	}

	//	Cross Product between 2 vectors
	inline void Cross(Vector3& a,Vector3& b)
	{
		x=a.y*b.z-a.z*b.y;
		y=a.z*b.x-a.x*b.z;
		z=a.x*b.y-a.y*b.x;
	}


	//	negation
	inline void Negate()
	{
		x=-x;
		y=-y;
		z=-z;
	}

    // access as array..
    inline float& operator[](unsigned  index) 
    {
        return v[index];
    }

	//	some overloads (single operand)
	Vector3& operator += ( const Vector3& in );
	Vector3& operator -= ( const Vector3& in );
	Vector3& operator *= ( const float& in );
	Vector3& operator /= ( const float& in );
	Vector3  operator -  (); // this is plain old negation

	/**
	* Constant vectors
	*/
	static const Vector3 Zero; // <0,0,0>
	static const Vector3 i; // <1,0,0>
	static const Vector3 j; // <0,1,0>
	static const Vector3 k; // <0,0,1>

};
/*-------------end of vector3 delcaration--------------*/




/********************************************************
*	vector3 public overloaded operator functions		*
********************************************************/

/**
* Accumulative addition of two vectors
*/
inline Vector3& Vector3::operator += ( const Vector3& in )
{
    x += in.x;   
    y += in.y;   
    z += in.z;
    return *this; // This is so we can chain operations together
}

/**
 * Accumulative subtraction of two vectors
 */
inline Vector3& Vector3::operator -= ( const Vector3& in )
{
	x -= in.x;   
	y -= in.y;   
	z -= in.z;
	return *this; // This is so we can chain operations together
}


/**
 * Accumulative multiplication of a vector by a scalar
 */
inline Vector3& Vector3::operator *= ( const float& in )
{
	x *= in;   
	y *= in;   
	z *= in;
	return *this; // This is so we can chain operations together
}


/**
 * Accumulative division of a vector by a scalar
 */
inline Vector3& Vector3::operator /= ( const float& in )
{
	float inv = 1.f / in; // do one (expensive) divide and 3 (cheap) mults
	x *= inv;   
	y *= inv;   
	z *= inv;
	return *this; // This is so we can chain operations together 
}

inline Vector3 Vector3::operator - () 
{
	return Vector3(
		-x,
		-y,
		-z
	);
}



//	addition
inline Vector3 operator+(const Vector3& p1,const Vector3& p2)
{
	return Vector3(p1.x+p2.x,p1.y+p2.y,p1.z+p2.z);
}

// subtraction
inline Vector3 operator-(const Vector3& p1,const Vector3& p2)
{
	return Vector3(p1.x-p2.x,p1.y-p2.y,p1.z-p2.z);
}

//	dot product
inline float operator*(const Vector3& p1,const Vector3& p2)
{
	return p1.x*p2.x+p1.y*p2.y+p1.z*p2.z;
}

//	cross product
inline Vector3 operator^(const Vector3& a,const Vector3& b)
{
	return Vector3
	(
		(a.y*b.z-a.z*b.y),
		(a.z*b.x-a.x*b.z),
		(a.x*b.y-a.y*b.x)
	);
}

// scaling.. the * operator	(by a float)
inline Vector3 operator*(const Vector3 &a,const float &s)
{
	return Vector3
		(
		a.x*s,
		a.y*s,
		a.z*s
		);
}

// scaling.. the * operator	(by a float)
inline Vector3 operator*(const float &s,const Vector3 &a)
{
	return Vector3
		(
		a.x*s,
		a.y*s,
		a.z*s
		);
}


// similarly.. division
inline Vector3 operator/(Vector3 const &a,float const &f)
{	
	float s=1.0f/f;
	return Vector3
		(
		a.x*s,
		a.y*s,
		a.z*s
		);
}


// overloading == eqaulity to check if 2 points are the same
inline bool operator==(Vector3 const &a,Vector3 const &b)
{
	if(fabs(a.x-b.x)<EPSILON)
	{
		if(fabs(a.y-b.y)<EPSILON)
		{
			if(fabs(a.z-b.z)<EPSILON)
			{
				return true;
			}
		}
	}
	return false;
}

// test for eqaulity using a sent epsilon
inline bool Vector3Equality(Vector3& a,Vector3& b,float& epsilon)
{
	if(fabs(a.x-b.x)<epsilon)
	{
		if(fabs(a.y-b.y)<epsilon)
		{
			if(fabs(a.z-b.z)<epsilon)
			{
				return true;
			}
		}
	}
	return false;
}

// those can be used to sort vertices differently
inline bool Vector3SortRuleOnX(Vector3& vertex1,Vector3& vertex2)
{
	return vertex1.x<vertex2.x;
}

// those can be used to sort vertices differently
inline bool Vector3SortRuleOnY(Vector3& vertex1,Vector3& vertex2)
{
	return vertex1.y<vertex2.y;
}

// those can be used to sort vertices differently
inline bool Vector3SortRuleOnZ(Vector3& vertex1,Vector3& vertex2)
{
	return vertex1.z<vertex2.z;
}


// angle between 2 3D vectors...
inline float GetVectorsAngle(Vector3 n1,Vector3 n2)
{
	n1.Normalize();
	n2.Normalize();
	return acosf(n1*n2);
}
/*-------------end of vector3 public overloads---------*/
#endif // _VECTOR3_H_