/********************************************************
*	vector4.h											*
*	By Shady El Mously									*
*	part of the twister game engine						*
*	this is the declaration file fo the vector4 struct	*
*	homogenous vector									*
********************************************************/

#ifndef _VECTOR4_H_
#define _VECTOR4_H_

#include "vector3.h"	//	we need vector3 here

/********************************************************
*	struct vector4 declaration							*
********************************************************/
struct Vector4
{
	union
	{
		float v[4];
		struct
		{
			float x,y,z,w;
		};
	};

	//	default constructor (empty)
	Vector4()		{}

	//	construct with 3 floats
	Vector4(float a,float b,float c,float d):x(a),y(b),z(c),w(d){}

	//	assigning values
	void Assign(float X,float Y,float Z,float W)
	{
		x=X;
		y=Y;
		z=Z;
		w=W;
	}

	//	treating vector4 as vector3 
	operator Vector3()
	{
		return Vector3(x,y,z);
	}

	//	an add function which is better than the + operator (no constructor called)
	void Add(const Vector3& p1,const Vector3& p2)
	{
		x=p1.x+p2.x;
		y=p1.y+p2.y;
		z=p1.z+p2.z;
	}
	
	//	a subtract function which is better than the + operator (no constructor called)
	void Subtract(const Vector3& p1,const Vector3& p2)
	{
		x=p1.x-p2.x;
		y=p1.y-p2.y;
		z=p1.z-p2.z;
	}

	//	get the magnitude squared of the vector
	float MagSquared()
	{
		return x*x+y*y+z*z;
	}

	//	magnitude of the vector
	float Mag()
	{	return sqrtf(x*x+y*y+z*z);	}

	//	making the vector a normal vector
	void Normalize()
	{
		float M=1.0f/sqrtf(x*x+y*y+z*z);
		assert(M && "vector magnitude is zero, cannot normalize!");
		x=x*M;
		y=y*M;
		z=z*M;
	}

	//	dot product..return a scalar quantity
	float Dot(Vector3& a)
	{	return x*a.x+y*a.y+z*a.z;	}

	//	Cross Product between 2 vectors
	void Cross(Vector3& a,Vector3& b)
	{
		x=a.y*b.z-a.z*b.y;
		y=a.z*b.x-a.x*b.z;
		z=a.x*b.y-a.y*b.x;
	}

	//	negation
	void Negate()
	{
		x=-x;
		y=-y;
		z=-z;
		w=-w;
	}

    // considering this is a quaternion
    // fill the 4x4 matrix
    void CreateMatrixFromQuaternion(float *pMatrix);

    // given a 3x3 or a 4x4 matrix
    // this lets this vector4 holds a quaternion
    void CreateQuaternionFromMatrix(float *pTheMatrix, int rowColumnCount);


    // considering this is a quaterion
    // this will perform a spherical linear interpolation
    Vector4 Slerp(Vector4 &q1, Vector4 &q2, float t);


	//	some overloads (single operand)
	Vector4& operator += ( const Vector3& in );
	Vector4& operator -= ( const Vector3& in );
	Vector4& operator *= ( const float& in );
	Vector4& operator /= ( const float& in );
	Vector4  operator -  (); // this is plain old negation
};
/*-------------end of vector4 declaration--------------*/


/********************************************************************
*	operator overloaded fcuntions for the Vector4 sructure			*
*	all are inline for assuring speed								*
********************************************************************/


inline const Vector4 operator+(Vector4 const &a, Vector4 const &b)
{
	return Vector4(a.x+b.x,a.y+b.y,a.z+b.z,a.w+b.w);
}; 



inline const Vector4 operator-(Vector4 const &a, Vector4 const &b)
{
	return Vector4(a.x-b.x,a.y-b.y,a.z-b.z,a.w-b.w);
}; 



inline const Vector4 operator*(Vector4 const &a, float const &b)
{
	return Vector4(a.x*b,a.y*b,a.z*b,a.w*b);
}; 



inline const Vector4 operator*(float  const &a, Vector4 const &b)
{
	return Vector4(a*b.x,a*b.y,a*b.z,a*b.w);
}; 



inline const Vector4 operator/(Vector4 const &a, float const &b)
{
	return Vector4(a.x/b,a.y/b,a.z/b,a.w/b);
}; 


inline float operator*(Vector4 const &a, Vector4 const &b)
{
	return a.x*b.x + a.y*b.y + a.z*b.z + a.w*b.w;
} 




#endif // _VECTOR4_H_

