#ifndef _TWISTER_MESH_
#define _TWISTER_MESH_
/********************************************************
*	mesh.h												*
* declaration file for the mesh interface				*
********************************************************/
#include "renderable.h"
#include "../hybrid/meshcompiler.h"
#include "../math/math.h"

/********************************************************
*	struct sMesh										*
*	represents a redareable 3d mesh						*
*	( acollection of vertices and textures				*
*	this is just an abstract base class					*
*	all 3d objects have a 3d mesh						*
*	which will be an object of class which implets the	*
*	interface defined here								*
********************************************************/
struct sMesh : public iRenderable
{
protected:

    // internal vertex buffer, never visible outside
    sVertexBuffer*  m_pVertexBuffer;

    // the number of allocated vertices inside it
    unsigned int    m_NumAllocatedVertices;

public:

    // where the mesh data starts
    float*          m_pVertexData;

    // the vertex format (content of each vertex)
    unsigned int    m_VertexFormat;

    // the render mode of the mesh
    unsigned int    m_RenderMode;

    // the number of vertices inside it
    unsigned int    m_NumVertices;

    // constructor
    sMesh();

    // destructor
    virtual ~sMesh();

    // create a simple mesh for testing purposes
    void CreateTestMesh();

    // create a sphere
    void CreateSphere(float radius,int numSegments=10,int numRings=10,float scaleX=1.0f,float scaleY=1.0f,float scaleZ=1.0f);

    // create a sphere of points
    void CreatePointsSphere(float radius,int numSegments=10,int numRings=10);

    // create a cylinder
    void CreateCylinder(float height,float bottomRadius,float topRadius,int numSegments=10);

    // create 3 axis with colors
    void CreateAxis(float length);

    // compile makes it ready for rendering
    void Compile();

    // render it
    inline void Render()
    {        m_pVertexBuffer->Render();    }

    // render a range of vertices as triangle fans
    inline void RenderFan(int startVertex,int numVerts)
    {        m_pVertexBuffer->RenderFan(startVertex,numVerts);    }

    // render a range of vertices as triangles
    inline void RenderTriangles(int startVertex,int numVerts)
    {        m_pVertexBuffer->RenderTriangles(startVertex,numVerts);    }

    // render a range of vertices as a triangle strip
    inline void RenderStrip(int startVertex,int numVerts)
    {        m_pVertexBuffer->RenderStrip(startVertex,numVerts);   }

    // begin using the mesh as vertex source
    inline void Enable()
    {
        m_pVertexBuffer->Enable();
    }

    // end using the mesh as vertex source
    inline void Disable()
    {
        m_pVertexBuffer->Disable();
    }

    // clean the mesh
    virtual void Clear();



    // this allocates a new number of vertices, copies the old ones inside it
    void Reserve(unsigned int numVerts);

    // this adds a new vertex, memory will be copied
    void AddVertex(float* pNewVertexData);

    // lock
    void Lock()
    {
        m_pVertexBuffer->Lock();
    }

    // unlock
    void Unlock()
    {
        m_pVertexBuffer->Unlock();
    }

    // this gets a vertex from inside 
    float* Getvertex(int index)
    {
        return m_pVertexBuffer->GetVertex(index);
    }

    // this creates a grid..like a terrain will Y zeros
    virtual void CreateGrid(float width,float depth,int numSegmentsX,int numSegmentsZ);

};
/*-----------------end of sMesh declaration------------*/
#endif // _TWISTER_MESH_