//=============================================================================
//
// 	Token.h   
//
// 	Author:	 James Boer
//
//	Copyright (C) 2001 James R. Boer  All Rights Reserved.
//	
//=============================================================================
#pragma once

#ifndef __TOKEN_H
#define __TOKEN_H

#define TOKEN_DEBUGGING_INFO

#pragma warning(disable:4786)

// Required for GUID definition
#include <windows.h>
#include <vector>
#include <list>
#include <assert.h>


// This is the basic data object generated by the Parser class
class Token 
{
public:
	enum TOKEN_TYPE
	{
		UNKNOWN_TOKEN = 0,
		KEYWORD,
		OPERATOR,
		VARIABLE,
		STRING,
		INTEGER,
		REAL,
		BOOLEAN,
		T_GUID
	};


	Token()						{  Clear();  }
	Token(const Token& tok);
	virtual ~Token()			{  Destroy();  }
	void Clear();

	void operator = ( const Token& tok);

	inline bool CreateKeyword(const char* pszKeyword)
	{
		type = KEYWORD;
		m_pszKeyword = new char[strlen(pszKeyword) + 1];
		if(!m_pszKeyword)
			return false;
		strcpy(m_pszKeyword, pszKeyword);
		return true;
	}

	inline bool CreateOperator(const char* pszOperator)
	{
		type = OPERATOR;
		m_pszOperator = new char[strlen(pszOperator) + 1];
		if(!m_pszOperator)
			return false;
		strcpy(m_pszOperator, pszOperator);
		return true;
	}

	inline bool CreateVariable(const char* pszVariable)
	{
		type = VARIABLE;
		m_pszVariable = new char[strlen(pszVariable) + 1];
		if(!m_pszVariable)
			return false;
		strcpy(m_pszVariable, pszVariable);
		return true;
	}

	inline bool CreateString(const char* pszString)
	{
		type = STRING;
		m_pszString = new char[strlen(pszString) + 1];
		if(!m_pszString)
			return false;
		strcpy(m_pszString, pszString);
		return true;
	}

	inline bool CreateInteger(int iInteger)
	{
		type = INTEGER;
		m_iInteger = iInteger;
		return true;
	}

	inline bool CreateReal(double dReal)
	{
		type = REAL;
		m_dReal = dReal;
		return true;
	}

	inline bool CreateBoolean(bool bBoolean)
	{
		type = BOOLEAN;
		m_bBoolean = bBoolean;
		return true;
	}

	inline bool CreateGuid(const GUID& guidGuid)
	{
		type = T_GUID;
		m_pguidGuid = new GUID;
		if(!m_pguidGuid)
			return false;
		memcpy(m_pguidGuid, &guidGuid, sizeof(GUID));
		return true;
	}

	void Destroy();

	inline void SetVariable(const char* pszVariable)
	{
		assert(type == VARIABLE);
		delete[] m_pszVariable;
		m_pszVariable = 0;
		CreateVariable(pszVariable);
	}

	inline void SetKeyword(const char* pszKeyword)
	{
		assert(type == KEYWORD);
		delete[] m_pszKeyword;
		m_pszKeyword = 0;
		CreateKeyword(pszKeyword);
	}

	inline void SetString(const char* pszString)
	{
		assert(type == STRING);
		delete[] m_pszString;
		m_pszString = 0;
		CreateString(pszString);
	}

	inline void SetOperator(const char* pszOperator)
	{
		assert(type == OPERATOR);
		delete[] m_pszOperator;
		m_pszOperator = 0;
		CreateOperator(pszOperator);
	}

	inline void SetInteger(const int iInteger)
	{
		assert(type == INTEGER);
		CreateInteger(iInteger);
	}

	inline void SetBoolean(const bool bBoolean)
	{
		assert(type == BOOLEAN);
		CreateBoolean(bBoolean);
	}

	inline void SetReal(const double dReal)
	{
		assert(type == REAL);
		CreateReal(dReal);
	}

	inline void SetGuid(const GUID& guidGuid)
	{
		assert(type == T_GUID);
		delete m_pguidGuid;
		m_pguidGuid = 0;
		CreateGuid(guidGuid);
	}

	// querry the type of token
	inline bool IsUnknown()	const		{  return (type == UNKNOWN_TOKEN) ? true : false;  }
	inline bool IsKeyword()	const		{  return (type == KEYWORD) ? true : false;  }
	inline bool IsOperator() const		{  return (type == OPERATOR) ? true : false;  }
	inline bool IsVariable() const		{  return (type == VARIABLE) ? true : false;  }
	inline bool IsString() const		{  return (type == STRING) ? true : false;  }
	inline bool IsInteger()	const		{  return (type == INTEGER) ? true : false;  }
	inline bool IsReal() const			{  return (type == REAL) ? true : false;  }
	inline bool IsBoolean()	const		{  return (type == BOOLEAN) ? true : false;  }
	inline bool IsGuid() const			{  return (type == T_GUID) ? true : false;  }
	inline TOKEN_TYPE GetType() const	{  return type;  }

	// get the token content
	inline const char* GetKeyword() const	{  assert(type == KEYWORD);  return (const char*)m_pszKeyword;  }
	inline const char* GetOperator() const	{  assert(type == OPERATOR);  return (const char*)m_pszOperator;  }
	inline const char* GetVariable() const	{  assert(type == VARIABLE);  return (const char*)m_pszVariable;  }
	inline const char* GetString() const	{  assert(type == STRING);  return (const char*)m_pszString;  }
	inline int GetInteger() const			{  assert(type == INTEGER);  return m_iInteger;  }
	inline double GetReal() const			{  assert(type == REAL);  return m_dReal;  }
	inline bool GetBoolean() const			{  assert(type == BOOLEAN);  return m_bBoolean;  }
	inline GUID& GetGuid() const			{  assert(type == T_GUID);  return *m_pguidGuid;  }

	inline bool IsNewline() const			{  return (IsOperator() && (GetOperator()[0] == '\n')) ? true : false;  }

	const char* GetDescriptiveString() const;
	
#ifdef TOKEN_DEBUGGING_INFO
	inline void SetLineNumber(int iLineNo)	{  m_iLineNumber = iLineNo;  }
	inline int GetLineNumber() const		{  return m_iLineNumber;  }
	inline void SetFileIndex(int iIndex)	{  m_iFileIndex = iIndex;  }
	inline int GetFileIndex() const			{  return m_iFileIndex;  }
#endif // TOKEN_DEBUGGING_INFO


protected:

	// This tells us which type of data we're carrying
	TOKEN_TYPE type;

	// The data is in one of these fields.  We're only going to be using
	// one type per token, so no reason to waste space.  So, we make
	// the data fields part of a union.
	union
	{
		char*			m_pszKeyword;
		char*			m_pszOperator;
		char*			m_pszVariable;
		char*			m_pszString;
		int				m_iInteger;
		double			m_dReal;
		bool			m_bBoolean;
		GUID*			m_pguidGuid;
	};

#ifdef TOKEN_DEBUGGING_INFO
	// Indicates the line number in which the token was found.
	// Note that although this is debugging information, it should
	// not only be contained in debug builds, since we may want to
	// "debug" release mode applications which still will be using
	// this parser.
	int m_iLineNumber;

	// References an index into a string table containing the source
	// file in which this token was found.
	int m_iFileIndex;
#endif // TOKEN_DEBUGGING_INFO

};

// stream operators
// this is to support a tokenized file representation of a parsed file
inline std::ostream& operator<<( std::ostream& os, Token& tok );
inline std::istream& operator>>( std::istream& is, Token& tok );


typedef std::vector<std::string> StringVector;
typedef std::vector<Token> TokenVector;
typedef std::vector<Token> TokenVector;

typedef std::list<Token>::iterator TokenListItor;

// TokenList is the standard list type for tokens.  It's a basic STL list
// of type token with some enhancements for storing and retrieving debugging info
class TokenList : public std::list<Token>
{
public:

	// Gets a descriptive string of the token including file name and line number
	std::string GetDescriptiveString(TokenListItor itr) const;
	// Gets a string of the file name from where the token was parsed
	//std::string GetFileName(TokenListItor itr) const;
	std::string GetFileName(int index) const;
	// Gets the line number of the token
	int GetLineNumber(TokenListItor itr) const;
	
	// Given a filename, find the index into the filelist vector.  If it doesn't
	// currently exist in the vector, push it onto the end and return the
	// new index value.
	int GetFileIndex(std::string sFileName);

	// Get the total number of file references in this token list
	int GetNumFileRefs() const;

	void clear();
	
private:
	StringVector m_aFileRef;
};



#endif // __TOKEN_H